<?php
// SwitchServer class
// (c) schmidi 2013
// www.doh-nuts.at


require_once('classes.inc.php');
require_once('SocketDataServer.php');
require_once('SocketDataFetcher.php');
require_once('PublicSocketDataServer.php');
require_once('PublicSocketDataFetcher.php');
require_once('FileDataServer.php');
require_once('FileDataFetcher.php');
require_once('SharedMemDataServer.php');
require_once('SharedMemDataFetcher.php');


	
abstract class SwitchServer {
	protected $debug = false;
	
	protected $manialinkId = 3487823;
	protected $openActionId = 3487821;
	protected $closeActionId = 3487822;
	
	protected $max_XML = '';
	protected $min_XML = '';
	
	protected $server_socket = false;
	protected $server_public_socket = false;
	protected $server_file = false;
	protected $server_shared_mem = false;

	protected $servers = array();
	protected $fetchers = array();
	protected $fetcher_data = array();
	
	protected $autohide;
	protected $autorestore;
	
	protected $user_min = array();
	protected $user_max = array();
	
	protected $tick = 10;
	
	protected $game_mode;
	protected $title_id;
	protected $running = true;
	
	protected $pos_ta;
	protected $pos_score_ta;
	protected $layout_ta;
	protected $pos_rounds;
	protected $pos_score_rounds;
	protected $layout_rounds;
	
	protected $position;
	protected $position_score;
	protected $min_layout;
	protected $max_layout;
	protected $server_layout;
	protected $score_layout;
	protected $score_server_layout;
	

	protected function loadSettings($file) {
		if (($xml = @simplexml_load_file($file)) === false) {
			$this->console('[SwitchServer] Could not read/parse config file ' . $file . ' !');
			die();
		}
		
		$this->autohide = $this->str2bool($xml->auto_hide);
		$this->autorestore = $this->str2bool($xml->auto_restore);
		
		$this->pos_ta = $xml->position_ta;
		$this->pos_score_ta = $xml->position_score_ta;
		$this->layout_ta = $xml->layout_ta;
		$this->pos_rounds = $xml->position_rounds;
		$this->pos_score_rounds = $xml->position_score_rounds;
		$this->layout_rounds = $xml->layout_rounds;
		
		
		// servers
		$this->server_socket = $this->str2bool($xml->servers->socket);
		if ($this->str2bool($xml->servers->public_socket)) {
			$this->server_public_socket = $xml->servers->public_socket->attributes();
		}	
		if ($this->str2bool($xml->servers->file)) {
			$this->server_file = $xml->servers->file;		
		}	
		if ($this->str2bool($xml->servers->shared_mem)) {
			$this->server_shared_mem = true;		
		}	
		
		
		// fetchers
		foreach ($xml->fetchers->children() as $child => $val) {
			switch (strtolower($child)) {
				case 'socket':
					$p = intval($val);
					if ($p) {
						$this->fetchers[] = new SocketDataFetcher('127.0.0.1', $p);
						$this->debug("added SocketDataFetcher port: {$p}");
					}
					break;

				case 'public_socket':
					$opt = $val->attributes();
					
					if ($opt["ip"] && intval($opt["port"]) && $opt["password"]) {
						$this->debug("added PublicSocketDataFetcher ip: {$opt['ip']}, port: {$opt['port']}, password: {$opt['password']}");
						$this->fetchers[] = new PublicSocketDataFetcher($opt["ip"], intval($opt["port"]), $opt["password"]);	
					}
					else {
						$this->debug("ERROR PublicSocketDataFetcher: no ip/port/password specified");
					}
					break;

				case 'file':
					if ($this->str2bool($val)) {
						$this->fetchers[] = new FileDataFetcher($val);		
						$this->debug("added FileDataFetcher file: {$val}");
					}
					break;

				case 'shared_mem':
					if ($this->str2bool($val)) {
						$this->fetchers[] = new SharedMemDataFetcher($val);		
						$this->debug("added SharedMemDataFetcher login: {$val}");
					}
					break;
			}		

		}
		
		$this->fetchData();
	}


	function init() {
		$this->setLayout();
		$this->generateMinXML();
		$this->generateMaxXML();
	}
		
	
	protected function setLayout() {
		if ($this->game_mode == 1 || $this->game_mode == 3 || $this->game_mode == 6) {
			$this->loadLayout(dirname(__FILE__) . '/' . $this->layout_rounds);
			$this->position = $this->pos_rounds;
			$this->position_score = $this->pos_score_rounds;
		}
		else {
			$this->loadLayout(dirname(__FILE__) . '/' . $this->layout_ta);
			$this->position = $this->pos_ta;
			$this->position_score = $this->pos_score_ta;
		}
	}
	
	
	protected function loadLayout($file) {
		if (($xml = @simplexml_load_file($file)) === false) {
			$this->console('[SwitchServer] Could not read/parse layout file ' . $file . ' !');
			die();
		}
		
		$this->min_layout = $xml->minimized->asXML();
		$this->max_layout = $xml->maximized->asXML();
		$this->server_layout = $xml->server->asXML();
		$this->score_layout = $xml->score->asXML();
		$this->score_server_layout = $xml->score_server->asXML();
	}
	

	protected function updateData($srv) {
		foreach ($this->servers as &$upd) {
			$upd->updateData($srv);
		}
	}
	
	
	protected function fetchData() {
		$this->fetcher_data = array();

		$this->debug('start fetching');
		foreach ($this->fetchers as &$srv) {
			$obj = $srv->fetchData();
			if ($obj !== false) {
				$this->fetcher_data[] = $obj;
				$this->debug('fetched: ' . $obj);
			}
			else {
				$this->debug('failed to fetch: ' . $srv);
			}
		}
		$this->debug('stop fetching');
	}
	
	
	protected function addMinUser($login) {
		$this->user_min[$login] = true;
		unset($this->user_max[$login]);
	}
	
	
	protected function addMaxUser($login) {
		unset($this->user_min[$login]);
		$this->user_max[$login] = true;
	}
	
	
	protected function generateXML() {
		$this->generateMinXML();
		$this->generateMaxXML();
	}
	
	
	protected function generateMinXML() {
		$CNT = max(1, count($this->fetcher_data));

		$xml = simplexml_load_string($this->min_layout);
		
		// replacement tokens
		$s = array();
		$s[] = '%POS%';
		$s[] = '%AID%';
		$r = array();
		$r[] = $this->position;
		$r[] = $this->openActionId;
		foreach ($xml->attributes() as $a => $b) {
			$s[] = "%{$a}%";
			$r[] = trim(eval("return {$b};"));
		}
		
		$this->min_XML = '<manialink id="' . $this->manialinkId . '">';
		$this->min_XML .= str_replace($s, $r, $xml->children()->asXML());
		$this->min_XML .= '</manialink>';	
	}
	

	protected function generateMaxXML() {
		$CNT = max(1, count($this->fetcher_data));

		// servers
		$i = 0;
		$srvs = '';
		foreach ($this->fetcher_data as &$srv) {
			$srvs .= $this->serverToXML($srv, $i++);
		}

		$xml = simplexml_load_string($this->max_layout);
		
		// replacement tokens
		$s = array();
		$s[] = '%POS%';
		$s[] = '%AID%';
		$s[] = '%SERVERS%';
		$r = array();
		$r[] = $this->position;
		$r[] = $this->closeActionId;
		$r[] = $srvs;
		foreach ($xml->attributes() as $a => $b) {
			$s[] = "%{$a}%";
			$r[] = trim(eval("return {$b};"));
		}
		
		$this->max_XML = '<manialink id="' . $this->manialinkId . '">';
		$this->max_XML .= str_replace($s, $r, $xml->children()->asXML());
		$this->max_XML .= '</manialink>';	
	}


	protected function getScoreXML() {
		$CNT = max(1, count($this->fetcher_data));

		// servers
		$I = 0;
		$srvs = '';
		foreach ($this->fetcher_data as &$SRV) {
			$xml = simplexml_load_string($this->score_server_layout);
		
			// replacement tokens
			$s = array();
			$r = array();
			foreach ($xml->attributes() as $a => $b) {
				$s[] = "%{$a}%";
				$r[] = eval("return {$b};");
			}
		
			$srvs .= str_replace($s, $r, $xml->children()->asXML());
			$I++;
		}

		// background
		$xml = simplexml_load_string($this->score_layout);
	
		// replacement tokens
		$s = array();
		$s[] = '%POS%';
		$s[] = '%SERVERS%';
		$r = array();
		$r[] = $this->position_score;
		$r[] = $srvs;
		foreach ($xml->attributes() as $a => $b) {
			$s[] = "%{$a}%";
			$r[] = trim(eval("return {$b};"));
		}

		$xmltxt = '<manialink id="' . $this->manialinkId . '">';	
		$xmltxt .= str_replace($s, $r, $xml->children()->asXML());
		$xmltxt .= '</manialink>';	
		
		return $xmltxt;
	}


	protected function serverToXML($SRV, $I) {
		$method = 'join';

		$xml = simplexml_load_string($this->server_layout);
		
		// replacement tokens
		foreach ($xml->attributes() as $a => $b) {
			$s[] = "%{$a}%";
			$r[] = trim(eval("return {$b};"));
		}
		
		return str_replace($s, $r, $xml->children()->asXML());
	}


	protected function stripLinks($input) {
		//thx to xymph and slig for replacement patterns

		return	str_replace("\0", '$$',
							preg_replace('/\\$[hlp](.*?)(?:\\[.*?\\](.*?))*(?:\\$[hlp]|$)/ixu', '$1$2',
								str_replace('$$', "\0", $input)));
	}


	protected function str2bool($str) {
		if (strtolower($str) == 'false' || $str == '0' || $str == '')
			return false;
		return true;
	}


	protected function console($txt) {
	}
	
	
	protected function debug($txt) {
		if ($this->debug) {
			$this->console('[switchServer] ' . $txt);
		}
	}
	
}	

?>